@@ -9,14 +9,17 @@
 #' - [parse_multipart()]: Parse `multipart/form-data` using [mime::parse_multipart()].
 #' - [parse_json()]: Parse `multipart/form-data` using [jsonlite::fromJSON()].
 #' 
-#' @return Returns the parsed value as a `list`.
+#' @return Returns the parsed value as a `list` or `NULL`
+#' if it failed to parse.
 #' 
 #' @name parsers
 #' @export 
-parse_multipart <- function(req, ...){
+parse_multipart <- function(req){
   check_installed("mime")  
-
-  mime::parse_multipart(req$body, ...)
+  tryCatch(
+    mime::parse_multipart(req$body),
+    error = function(e) NULL
+  )
 }
 
 #' @export 
@@ -24,5 +27,8 @@ parse_multipart <- function(req, ...){
 parse_json <- function(req, ...){
   data <- req$body[["rook.input"]]
   data <- data$read_lines()
-  jsonlite::fromJSON(data, ...)
+  tryCatch(
+    jsonlite::fromJSON(data, ...),
+    error = function(e) NULL
+  )
 }
