@@ -25,26 +25,49 @@
 #' @name responses
 #'
 #' @export
-response <- function(body, headers = list('Content-Type' = 'text/html'), status = 200L){
+response <- function(body, headers = list(), status = 200L){
   assert_that(not_missing(body))
-  res <- list(status = as.integer(status), headers = headers, body = as.character(body))
+  res <- list(status = as.integer(status), headers = headers, body = convert_body(body))
   construct_response(res)
 }
 
 #' @rdname responses
 #' @export
-response_404 <- function(body = "404: Not found", headers = list('Content-Type' = 'text/html'), status = 404L){
-  res <- list(status = as.integer(status), headers = headers, body = as.character(body))
+response_404 <- function(body = "404: Not found", headers = list("Content-Type" = content_html()), status = 404L){
+  res <- list(status = as.integer(status), headers = headers, body = convert_body(body))
   construct_response(res)
 }
 
 #' @rdname responses
 #' @export
-response_500 <- function(body = "500: Server Error", headers = list('Content-Type' = 'text/html'), status = 500L){
-  res <- list(status = as.integer(status), headers = headers, body = as.character(body))
+response_500 <- function(body = "500: Server Error", headers = list("Content-Type" = content_html()), status = 500L){
+  res <- list(status = as.integer(status), headers = headers, body = convert_body(body))
   construct_response(res)
 }
 
+#' Convert Body
+#' 
+#' Body may only be a character vector of length 1.
+#' 
+#' @param body Body of response.
+#' 
+#' @keywords internal
+convert_body <- function(body) {
+  if(inherits(body, "AsIs"))
+    return(body)
+
+  if(is.raw(body))
+    return(body)
+
+  if(is.character(body) && length(body) == 1L)
+    return(body)
+
+  if(is.factor(body) || inherits(body, "shiny.tag"))
+    return(as.character(body))
+
+  return(body)
+}
+
 #' Construct Response
 #'
 #' @noRd
@@ -58,9 +81,17 @@ print.ambiorixResponse <- function(x, ...){
   cat("An ambiorix response")
 }
 
+#' @keywords internal
+is_response <- function(obj) {
+  inherits(obj, "ambiorixResponse")
+}
+
 #' Response
 #' 
 #' Response class to generate responses sent from the server.
+#' 
+#' @field status Status of the response, defaults to `200L`.
+#' @field headers Named list of headers.
 #'
 #' @export 
 Response <- R6::R6Class(
@@ -69,7 +100,7 @@ Response <- R6::R6Class(
   public = list(
 #' @details Set the status of the response.
 #' @param status An integer defining the status.
-    status = function(status){
+    set_status = function(status){
       assert_that(not_missing(status))
       private$.status <- status
       invisible(self)
@@ -78,54 +109,66 @@ Response <- R6::R6Class(
 #' @param body Body of the response.
 #' @param headers HTTP headers to set.
 #' @param status Status of the response, if `NULL` uses `self$status`.
-    send = function(body, headers = list('Content-Type' = 'text/html'), status = NULL){
+    send = function(body, headers = NULL, status = NULL){
+      deprecated_headers(headers)
+      deprecated_status(status)
       headers <- private$.get_headers(headers)
-      response(status = private$.get_status(status), headers = headers, body = as.character(body))
+      response(status = private$.get_status(status), headers = headers, body = convert_body(body))
     },
 #' @details Send a plain HTML response, pre-processed with sprintf.
 #' @param body Body of the response.
 #' @param ... Passed to `...` of `sprintf`.
 #' @param headers HTTP headers to set.
 #' @param status Status of the response, if `NULL` uses `self$status`.
-    sendf = function(body, ..., headers = list('Content-Type' = 'text/html'), status = NULL){
+    sendf = function(body, ..., headers = NULL, status = NULL){
+      deprecated_headers(headers)
+      deprecated_status(status)
       body <- sprintf(body, ...)
       headers <- private$.get_headers(headers)
-      response(status = private$.get_status(status), headers = headers, body = as.character(body))
+      response(status = private$.get_status(status), headers = headers, body = convert_body(body))
     },
 #' @details Send a plain text response.
 #' @param body Body of the response.
 #' @param headers HTTP headers to set.
 #' @param status Status of the response, if `NULL` uses `self$status`.
-    text = function(body, headers = list('Content-Type' = 'text/plain'), status = NULL){
+    text = function(body, headers = NULL, status = NULL){
+      deprecated_headers(headers)
+      deprecated_status(status)
       headers <- private$.get_headers(headers)
-      response(status = private$.get_status(status), headers = headers, body = as.character(body))
+      headers[["Content-Type"]] <- content_plain()
+      response(status = private$.get_status(status), headers = headers, body = convert_body(body))
     },
 #' @details Send a file.
 #' @param file File to send.
 #' @param headers HTTP headers to set.
 #' @param status Status of the response.
-    send_file = function(file, headers = list('Content-Type' = 'text/html'), status = NULL){
+    send_file = function(file, headers = NULL, status = NULL){
+      deprecated_headers(headers)
+      deprecated_status(status)
       assert_that(not_missing(file))
-      self$render(file, data = list(), status = private$.get_status(status))
+      self$render(file, data = list(), status = status, headers = headers)
     },
 #' @details Redirect to a path or URL.
 #' @param path Path or URL to redirect to.
 #' @param status Status of the response, if `NULL` uses `self$status`.
     redirect = function(path, status = NULL){
+      deprecated_status(status)
       status <- private$.get_status(status)
-      if(!grepl("^3", status))
-        status <- 302L
-
-      response(status = status, headers = list(Location = path), body = "")
+      headers <- private$.get_headers(list(Location = path))
+      response(status = status, headers = headers, body = "")
     },
 #' @details Render a template file.
 #' @param file Template file.
 #' @param data List to fill `[% tags %]`.
 #' @param headers HTTP headers to set.
 #' @param status Status of the response, if `NULL` uses `self$status`.
-    render = function(file, data = list(), headers = list('Content-Type' = 'text/html'), status = NULL){
+    render = function(file, data = list(), headers = NULL, status = NULL){
       assert_that(not_missing(file))
       assert_that(has_file(file))
+      deprecated_status(status)
+      deprecated_headers(headers)
+
+      status <- private$.get_status(status)
 
       file_content <- private$.render_template(file, data)
       headers <- private$.get_headers(headers)
@@ -137,10 +180,12 @@ Response <- R6::R6Class(
 #' @param headers HTTP headers to set.
 #' @param status Status of the response, if `NULL` uses `self$status`.
 #' @param ... Additional arguments passed to the serialiser.
-    json = function(body, headers = list("Content-Type" = "application/json"), status = NULL, ...){
-      to_json <- get_serialise(...)
+    json = function(body, headers = NULL, status = NULL, ...){
+      self$header_content_json()
+      deprecated_headers(headers)
+      deprecated_status(status)
       headers <- private$.get_headers(headers)
-      response(to_json(body), headers = headers, status = private$.get_status(status))
+      response(serialise(body), headers = headers, status = private$.get_status(status))
     },
 #' @details Sends a comma separated value file
 #' @param data Data to convert to CSV.
@@ -150,13 +195,14 @@ Response <- R6::R6Class(
     csv = function(data, name = "data", status = NULL, ...){
       assert_that(not_missing(data))
       check_installed("readr")
+      deprecated_status(status)
 
       name <- sprintf("attachment;charset=UTF-8;filename=%s.csv", name)
 
       headers <- list(
-        "Content-Type" = "text/csv",
         "Content-Disposition" = name
       )
+      self$header_content_csv()
       headers <- private$.get_headers(headers)
 
       data <- readr::format_csv(data, ...)
@@ -170,13 +216,14 @@ Response <- R6::R6Class(
     tsv = function(data, name = "data", status = NULL, ...){
       assert_that(not_missing(data))
       check_installed("readr")
+      deprecated_status(status)
 
       name <- sprintf("attachment;charset=UTF-8;filename=%s.tsv", name)
 
       headers <- list(
-        "Content-Type" = "tab-separated-values",
         "Content-Disposition" = name
       )
+      self$header_content_tsv()
       headers <- private$.get_headers(headers)
 
       data <- readr::format_tsv(data, ...)
@@ -190,11 +237,15 @@ Response <- R6::R6Class(
       check_installed("htmlwidgets")
       if(!inherits(widget, "htmlwidget"))
         stop("This is not an htmlwidget", call. = FALSE)
+      deprecated_status(status)
 
       # save and read
       tmp <- tempfile(fileext = ".html")
       htmlwidgets::saveWidget(widget, tmp, selfcontained = TRUE, ...)
-      headers <- private$.get_headers(headers)
+      on.exit({
+        unlink(tmp)
+      })
+      headers <- private$.get_headers()
 
       response(body = paste0(read_lines(tmp), "\n", collapse = ""), status = private$.get_status(status), headers = headers)
     },
@@ -203,17 +254,49 @@ Response <- R6::R6Class(
 #' @param data List to fill `[% tags %]`.
 #' @param headers HTTP headers to set.
 #' @param status Status of the response, if `NULL` uses `self$status`.
-    md = function(file, data = list(), headers = list('Content-Type' = 'text/html'), status = NULL) {
+    md = function(file, data = list(), headers = NULL, status = NULL) {
       check_installed("commonmark")
+      deprecated_headers(headers)
+      deprecated_status(status)
       self$render(file, data, headers, status)
     },
-#' @details Add headers to the response.
-#' @param name,value Name and value of the header.
-#' @return Invisibly returns self.
-    header = function(name, value){
-      name <- as_label(name)
-      private$.headers[[name]] <- value
-      invisible(self)
+#' @details Send a png file
+#' @param file Path to local file.
+    png = function(file){
+      private$.send_image(file, "png")
+    },
+#' @details Send a jpeg file
+#' @param file Path to local file.
+    jpeg = function(file) {
+      private$.send_image(file, "jpeg")
+    },
+#' @details Send an image
+#' Similar to `png` and `jpeg` methods but guesses correct method 
+#' based on file extension.
+#' @param file Path to local file.
+    image = function(file) {
+      type <- tools::file_ext(file)
+      if(!type %in% c("png", "jpeg"))
+        stop("Only accepts .png and .jpeg files")
+      private$.send_image(file, type)
+    },
+#' @details Ggplot2
+#' @param plot Ggplot2 plot object.
+#' @param type Type of image to save.
+#' @param ... Passed to [ggplot2::ggsave()]
+    ggplot2 = function(plot, ..., type = c("png", "jpeg")) {
+      assert_that(not_missing(plot))
+      check_installed("ggplot2")
+
+      type <- match.arg(type)
+      ext <- sprintf(".%s", type)
+      temp <- tempfile(fileext = ext)
+      ggplot2::ggsave(
+        temp,
+        plot, 
+        ...
+      ) 
+      private$.send_image(temp, type, clean = TRUE)
     },
 #' @details Print
     print = function(){
@@ -239,6 +322,11 @@ Response <- R6::R6Class(
     set = function(name, value){
       assert_that(not_missing(name))
       assert_that(not_missing(value))
+      .Deprecated(
+        "",
+        package = "ambiorix",
+        "Deprecated. The environment is no longer locked, you may simply `res$name <- value`"
+      )
 
       name <- as_label(name)
       self[[name]] <- value
@@ -249,15 +337,94 @@ Response <- R6::R6Class(
 #' @param name Name of the variable to get.
     get = function(name){
       assert_that(not_missing(name))
+      .Deprecated(
+        "",
+        package = "ambiorix",
+        "Deprecated. The environment is no longer locked, you may simply `req$value"
+      )
 
       name <- as_label(name)
       self[[name]]
     },
+#' @details Add headers to the response.
+#' @param name,value Name and value of the header.
+#' @return Invisibly returns self.
+    header = function(name, value){
+      assert_that(not_missing(name))
+      assert_that(not_missing(value))
+      name <- as_label(name)
+      private$.headers[[name]] <- value
+      invisible(self)
+    },
+#' @details Set Content Type to JSON
+#' @return Invisibly returns self.
+    header_content_json = function(){
+      self$header("Content-Type", content_json())
+      invisible(self)
+    },
+  #' @details Set Content Type to HTML
+  #' @return Invisibly returns self.
+    header_content_html = function(){
+      self$header("Content-Type", content_html())
+      invisible(self)
+    },
+  #' @details Set Content Type to Plain Text
+  #' @return Invisibly returns self.
+    header_content_plain = function(){
+      self$header("Content-Type", content_plain())
+      invisible(self)
+    },
+  #' @details Set Content Type to CSV
+  #' @return Invisibly returns self.
+    header_content_csv = function(){
+      self$header("Content-Type", content_csv())
+      invisible(self)
+    },
+  #' @details Set Content Type to TSV
+  #' @return Invisibly returns self.
+    header_content_tsv = function(){
+      self$header("Content-Type", content_tsv())
+      invisible(self)
+    },
 #' @details Get headers
 #' Returns the list of headers currently set.
     get_headers = function() {
       return(private$.headers)
     },
+#' @details Get a header
+#' Returns a single header currently, `NULL` if not set.
+#' @param name Name of the header to return.
+    get_header = function(name) {
+      assert_that(not_missing(name))
+      return(private$.headers[[name]])
+    },
+#' @details Set headers
+#' @param headers A named list of headers to set.
+    set_headers = function(headers) {
+      assert_that(not_missing(headers))
+      if(!is.list(headers))
+        stop("`headers` must be a named list")
+
+      private$.headers <- headers
+      invisible(self)
+    },
+    #' @details Set a Header
+    #' @param name Name of the header.
+    #' @param value Value to set.
+    #' @return Invisible returns self.
+    set_header = function(name, value) {
+      assert_that(not_missing(name))
+      assert_that(not_missing(value))
+
+      .Deprecated(
+        "header",
+        package = "ambiorix",
+        "Deprecated. This is deprecated, use the `header()` method."
+      )
+
+      private$.headers[[name]] <- value
+      invisible(self)
+    },
 #' @details Add a pre render hook.
 #' Runs before the `render` and `send_file` method.
 #' 
@@ -322,10 +489,12 @@ Response <- R6::R6Class(
       invisible(self)
     },
 #' @details Set a cookie
+#' Overwrites existing cookie of the same `name`.
 #' @param name Name of the cookie.
 #' @param value value of the cookie.
 #' @param expires Expiry, if an integer assumes it's the number of seconds
 #' from now. Otherwise accepts an object of class `POSIXct` or `Date`.
+#' If a `character` string then it is set as-is and not pre-processed.
 #' If unspecified, the cookie becomes a session cookie. A session finishes 
 #' when the client shuts down, after which the session cookie is removed. 
 #' @param max_age Indicates the number of seconds until the cookie expires. 
@@ -340,87 +509,100 @@ Response <- R6::R6Class(
 #' request is made with the https: scheme (except on localhost), and therefore, 
 #' is more resistant to man-in-the-middle attacks.
 #' @param http_only Forbids JavaScript from accessing the cookie, for example,
-#' through the Document.cookie property.
+#' through the document.cookie property.
 #' @param same_site Controls whether or not a cookie is sent with cross-origin
 #' requests, providing some protection against cross-site request forgery
 #' attacks (CSRF). Accepts `Strict`, `Lax`, or `None`.
+#' @return Invisibly returns self.
     cookie = function(
       name,
       value,
-      expires = NULL,
-      max_age = NULL,
-      domain = NULL,
-      path = NULL,
-      secure = TRUE,
-      http_only = FALSE,
-      same_site = NULL
+      expires = getOption("ambiorix.cookie.expire"),
+      max_age = getOption("ambiorix.cookie.maxage"),
+      domain = getOption("ambiorix.cookie.domain"),
+      path = getOption("ambiorix.cookie.path", "/"),
+      secure = getOption("ambiorix.cookie.secure", TRUE),
+      http_only = getOption("ambiorix.cookie.httponly", TRUE),
+      same_site = getOption("ambiorix.cookie.savesite")
     ) {
       assert_that(not_missing(name))
       assert_that(not_missing(value))
 
+      if(length(.globals$cookiePreprocessors) > 0) {
+        for(i in 1:length(.globals$cookiePreprocessors)) {
+          value <- .globals$cookiePreprocessors[[i]](
+            name,
+            value,
+            expires,
+            max_age,
+            domain,
+            path,
+            secure,
+            http_only,
+            same_site
+          )
+        }
+      }
+
       name <- as_label(name)
-      cookie <- sprintf("%s=%s", name, value)
+      private$.cookies[[name]] <- cookie(
+        name,
+        value,
+        expires,
+        max_age,
+        domain,
+        path,
+        secure,
+        http_only,
+        same_site
+      )
 
-      if(!is.null(expires)) {
-        expires <- convert_cookie_expires(expires)
-        cookie <- sprintf("%s; Expires=%s", cookie, expires)
-      }
-
-      if(!is.null(max_age)) {
-        cookie <- sprintf("%s; Max-Age=%s", cookie, max_age)
-      }
-
-      if(!is.null(domain)) {
-        cookie <- sprintf("%s; Domain=%s", cookie, domain)
-      }
-
-      if(!is.null(path)) {
-        cookie <- sprintf("%s; Path=%s", cookie, path)
-      }
-
-      if(secure) {
-        cookie <- sprintf("%s; Secure", cookie)
-      }
-
-      if(http_only) {
-        cookie <- sprintf("%s; HttpOnly", cookie)
-      }
-
-      if(!is.null(same_site)) {
-        cookie <- sprintf("%s; SameSite=%s", cookie, same_site)
-      }
-
-      private$.headers[["Set-Cookie"]] <- cookie
+      invisible(self)
+    },
+#' @details Clear a cookie
+#' Clears the value of a cookie.
+#' @param name Name of the cookie to clear.
+#' @return Invisibly returns self.
+    clear_cookie = function(name) {
+      # cookies with date in the past are removed from the browser
+      self$cookie(
+        name,
+        "",
+        expires = Sys.Date() - 365L
+      )
 
       invisible(self)
     }
   ),
+  active = list(
+    status = function(value) {
+      if(missing(value))
+        return(private$.status)
+
+      private$.status <- as.integer(value)
+    },
+    headers = function(value) {
+      if(missing(value))
+        return(private$.headers)
+
+      if(!is.list(value))
+        stop("Must be a `list`")
+
+      private$.headers <- value
+    }
+  ),
   private = list(
-    .templates = list(),
     .status = 200L,
+    .cookies = list(),
     .headers = list(), 
     .preHooks = list(),
     .postHooks = list(),
-    .get_template_path = function(file){
-      file <- remove_extensions(file)
-
-      # should be recursive?
-      # try HTML first
-      file_html <- private$.try_template_path(file, ".html")
-
-      if(!is.null(file_html)) return(file_html)
-
-      # try R
-      file_r <- private$.try_template_path(file, ".R")
-
-      if(is.null(file_r)){
-        msg <- sprintf("Cannot find %s.html nor %s.R in templates", file, file)
-        stop(msg)
-      }
-
-      return(file_r)
-    },
     .render_template = function(file, data){
+      file <- normalizePath(file)
+
+      if(!is.null(.globals$renderer))
+        return(.globals$renderer(file, data))
+
       # read and replace tags
       file_content <- read_lines(file)
 
@@ -429,43 +611,24 @@ Response <- R6::R6Class(
 
       # handle partials
       # replace brackets so glue::glue_data evals
-      file_content <- replace_partials(file, file_content, ext = ext)
+      file_content <- replace_partials(file_content, get_dir(file))
 
-      if(ext == "md")
-        file_content <- commonmark::markdown_html(file_content)
-
-      if(ext == "html"){
-
-        # needs serialisation
-        to_json <- get_serialise()
-
-        # serialise to each object individually
-        data <- lapply(data, function(x){
-          to_json(x)
-        })
-      } 
-
-      if(ext != "html")
-        data <- lapply(data, function(x){
-
-          # If not AsIs can use object
-          if(!inherits(x, "robj"))
+      if(ext == "html") {
+        data <- lapply(data, \(x) {
+          if(!inherits(x, "jobj"))
             return(x)
 
-          paste0(
-            capture.output(
-              dput(x)
-            ),
-            collapse = ""
-          )
+          serialise(x) |> 
+            as.character()
         })
+      }
 
       # hooks
       if(length(private$.preHooks) > 0) {
         for(i in 1:length(private$.preHooks)) {
           pre_processed <- private$.preHooks[[i]](self, file_content, data, ext)
-          if(!inherits(pre_processed, "responsePreHook")){
-            cat(error(), "Not a valid response from pre-hook (ignoring)\n")
+          if(!is_pre_hook(pre_processed)){
+            cat(error(), "Not a valid return value from pre-hook (ignoring)\n")
             next
           }
 
@@ -474,9 +637,10 @@ Response <- R6::R6Class(
         }
       }
 
-      file_content <- lapply(file_content, function(x, data){
-        glue::glue_data(data, x, .open = "[%", .close = "%]")
-      }, data = data)
+      file_content <- render_tags(file_content, data)
+      
+      if(ext == "md")
+        file_content <- commonmark::markdown_html(file_content)
 
       # collapse html
       if(ext == "html" || ext == "md")
@@ -485,24 +649,23 @@ Response <- R6::R6Class(
       # parse R
       private$.run_post_hooks(render_html(file_content), ext)
     },
-    .make_template_path = function(file){
-      # clean input
-      file <- remove_extensions(file)
-
-      # list possible files
-      files <- remove_extensions(private$.templates)
-
-      template <- sprintf("./templates/%s.html", template)
-      normalizePath(template)
-    },
     .get_status = function(status){
       if(is.null(status))
         return(private$.status)
 
       status
     },
-    .get_headers = function(headers = list()){
-      modifyList(private$.headers, headers)
+    .get_headers = function(headers = NULL){
+      private$.render_cookies()
+      heads <- private$.headers
+
+      if(!is.null(headers))
+        heads <- modifyList(heads, headers)
+
+      if(is.null(heads[["Content-Type"]]))
+        heads <- modifyList(heads, list("Content-Type" = content_html()))
+
+      return(heads)
     },
     .run_post_hooks = function(file_content, ext) {
       if(length(private$.postHooks) == 0) {
@@ -513,7 +676,7 @@ Response <- R6::R6Class(
         content <- private$.postHooks[[i]](self, file_content, ext)
 
         if(!is.character(content)){
-          cat(error(), "Not a character response from post-hook (ignoring)\n", stdout())
+          cat(error(), "Not a character vector returned from post-hook (ignoring)\n", stdout())
           next
         }
 
@@ -521,60 +684,75 @@ Response <- R6::R6Class(
       }
 
       return(file_content)
+    },
+    .render_cookies = function() {
+      if(length(private$.cookies) == 0L)
+        return()
+
+      for(opts in private$.cookies) {
+        cookie <- sprintf("%s=%s", opts$name, opts$value)
+
+        if(!is.null(opts$expires)) {
+          expires <- convert_cookie_expires(opts$expires)
+          cookie <- sprintf("%s; Expires=%s", cookie, expires)
+        }
+
+        if(!is.null(opts$max_age))
+          cookie <- sprintf("%s; Max-Age=%s", cookie, opts$max_age)
+
+        if(!is.null(opts$domain))
+          cookie <- sprintf("%s; Domain=%s", cookie, opts$domain)
+
+        if(!is.null(opts$path))
+          cookie <- sprintf("%s; Path=%s", cookie, opts$path)
+
+        if(opts$secure)
+          cookie <- sprintf("%s; Secure", cookie)
+
+        if(opts$http_only)
+          cookie <- sprintf("%s; HttpOnly", cookie)
+
+        if(!is.null(opts$same_site)) 
+          cookie <- sprintf("%s; SameSite=%s", cookie, opts$same_site)
+
+        names(cookie) <- "Set-Cookie"
+
+        private$.headers <- append(
+          private$.headers,
+          as.list(cookie)
+        )
+      }
+    },
+    .send_image = function(file, type = c("png", "jpeg"), clean = FALSE) {
+      assert_that(not_missing(file))
+
+      if(grepl("http", file))
+        stop("Must be a local file", call. = FALSE)
+
+      if(clean) {
+        on.exit({
+          unlink(file)
+        })
+      }
+
+      type <- match.arg(type)
+      type <- sprintf("image/%s", type)
+
+      size <- file.info(file)$size
+      con <- file(file, "rb")
+      on.exit({
+        close(con)
+      }, add = TRUE)
+
+      raw <- readBin(con, raw(), size)
+
+      self$header("Content-Length", size)
+      self$header("Content-Type", type)
+      self$send(raw)
     }
   )
 )
 
-#' Data Object
-#'
-#' Treats a data element rendered in a response (`res$render`) as
-#' a data object and ultimately uses [dput()].
-#'
-#' For instance in a template, `x <- [% var %]` will not work with
-#' `res$render(data=list(var = "hello"))` because this will be replace
-#' like `x <- hello` (missing quote): breaking the template. Using `robj` one would
-#' obtain `x <- "hello"`.
-#'
-#' @param obj R object to treat.
-#'
-#' @export
-robj <- function(obj){
-  assert_that(not_missing(obj))
-
-  # Supress warnings otherwise
-  # NULL, NA, and the likes
-  # raise messages
-  suppressWarnings(
-    structure(obj, class = c("robj", class(obj)))
-  )
-}
-
-#' @export
-print.robj <- function(x, ...){
-  cli::cli_alert_info("R object")
-  class(x) <- class(x)[class(x) != "robj"]
-  print(x)
-}
-
-#' Pre Hook Response
-#' 
-#' @param content File content, a character vector.
-#' @param data A list of data passed to `glue::glue_data`.
-#' 
-#' @export 
-pre_hook <- function(
-  content,
-  data
-) {
-  structure(
-    list(
-      content = content,
-      data = data
-    ),
-    class = c("list", "responsePreHook")
-  )
-}
-
 #' Convert Cookie Expires
 #' 
 #' Converts the cookie `expires` argument
@@ -614,3 +792,25 @@ convert_cookie_expires <- function(expires) {
 
   return(expires)
 }
+
+deprecated_headers <- function(headers = NULL) {
+  if(is.null(headers))
+    return()
+
+  .Deprecated(
+    "header",
+    package = "ambiorix",
+    msg = "Deprecated. Pass headers with the `header()` method."
+  )
+}
+
+deprecated_status <- function(status = NULL) {
+  if(is.null(status))
+    return()
+
+  .Deprecated(
+    "status",
+    package = "ambiorix",
+    msg = "Deprecated. Pass status to the `status` binding, e.g.: `res$status <- 404L`."
+  )
+}
\ No newline at end of file
